@@ -323,7 +323,8 @@ void init_max_user_conn(void)
 {
 #ifndef NO_EMBEDDED_ACCESS_CHECKS
   my_hash_init(key_memory_user_conn, &hash_user_connections,
-               system_charset_info, max_connections, 0, 0, (my_hash_get_key)
+               USER_CONN::user_host_key_charset_info_for_hash(),
+               max_connections, 0, 0, (my_hash_get_key)
                get_key_conn, (my_hash_free_key) free_user, 0);
 #endif
 }
@@ -429,10 +430,7 @@ void init_user_stats(USER_STATS *user_stats,
                      ulonglong bytes_sent,
                      ulonglong binlog_bytes_written,
                      ha_rows rows_sent,
-                     ha_rows rows_read,
-                     ha_rows rows_inserted,
-                     ha_rows rows_deleted,
-                     ha_rows rows_updated,
+                     rows_stats *rows_stats,
                      ulonglong select_commands,
                      ulonglong update_commands,
                      ulonglong other_commands,
@@ -463,10 +461,7 @@ void init_user_stats(USER_STATS *user_stats,
   user_stats->bytes_sent= bytes_sent;
   user_stats->binlog_bytes_written= binlog_bytes_written;
   user_stats->rows_sent= rows_sent;
-  user_stats->rows_read= rows_read;
-  user_stats->rows_inserted= rows_inserted;
-  user_stats->rows_deleted= rows_deleted;
-  user_stats->rows_updated= rows_updated;
+  user_stats->rows_stats= *rows_stats;
   user_stats->select_commands= select_commands;
   user_stats->update_commands= update_commands;
   user_stats->other_commands= other_commands;
@@ -483,14 +478,18 @@ void init_user_stats(USER_STATS *user_stats,
 
 void init_global_user_stats(void)
 {
-  my_hash_init(PSI_INSTRUMENT_ME, &global_user_stats, system_charset_info, max_connections,
+  my_hash_init(PSI_INSTRUMENT_ME, &global_user_stats,
+               USER_STATS::user_key_charset_info_for_hash(),
+               max_connections,
                0, 0, (my_hash_get_key) get_key_user_stats,
                (my_hash_free_key) free_user_stats, 0);
 }
 
 void init_global_client_stats(void)
 {
-  my_hash_init(PSI_INSTRUMENT_ME, &global_client_stats, system_charset_info, max_connections,
+  my_hash_init(PSI_INSTRUMENT_ME, &global_client_stats,
+               USER_STATS::user_key_charset_info_for_hash(),
+               max_connections,
                0, 0, (my_hash_get_key) get_key_user_stats,
                (my_hash_free_key) free_user_stats, 0);
 }
@@ -571,6 +570,8 @@ static bool increment_count_by_name(const char *name, size_t name_length,
   if (!(user_stats= (USER_STATS*) my_hash_search(users_or_clients, (uchar*) name,
                                               name_length)))
   {
+    struct rows_stats rows_stats;
+    bzero(&rows_stats, sizeof(rows_stats));
     /* First connection for this user or client */
     if (!(user_stats= ((USER_STATS*)
                        my_malloc(PSI_INSTRUMENT_ME, sizeof(USER_STATS),
@@ -581,8 +582,8 @@ static bool increment_count_by_name(const char *name, size_t name_length,
                     0, 0, 0,   // connections
                     0, 0, 0,   // time
                     0, 0, 0,   // bytes sent, received and written
-                    0, 0,      // rows sent and read
-                    0, 0, 0,   // rows inserted, deleted and updated
+                    0,
+                    &rows_stats,
                     0, 0, 0,   // select, update and other commands
                     0, 0,      // commit and rollback trans
                     thd->status_var.access_denied_errors,
@@ -673,16 +674,22 @@ static void update_global_user_stats_with_user(THD *thd,
     (thd->status_var.binlog_bytes_written -
      thd->org_status_var.binlog_bytes_written);
   /* We are not counting rows in internal temporary tables here ! */
-  user_stats->rows_read+=      (thd->status_var.rows_read -
-                                thd->org_status_var.rows_read);
-  user_stats->rows_sent+=      (thd->status_var.rows_sent -
-                                thd->org_status_var.rows_sent);
-  user_stats->rows_inserted+=  (thd->status_var.ha_write_count -
-                                thd->org_status_var.ha_write_count);
-  user_stats->rows_deleted+=   (thd->status_var.ha_delete_count -
-                                thd->org_status_var.ha_delete_count);
-  user_stats->rows_updated+=   (thd->status_var.ha_update_count -
-                                thd->org_status_var.ha_update_count);
+  user_stats->rows_sent+=            (thd->status_var.rows_sent -
+                                      thd->org_status_var.rows_sent);
+  user_stats->rows_stats.read+=      (thd->status_var.rows_read -
+                                      thd->org_status_var.rows_read);
+  user_stats->rows_stats.inserted+=  (thd->status_var.ha_write_count -
+                                      thd->org_status_var.ha_write_count);
+  user_stats->rows_stats.deleted+=   (thd->status_var.ha_delete_count -
+                                      thd->org_status_var.ha_delete_count);
+  user_stats->rows_stats.updated+=   (thd->status_var.ha_update_count -
+                                      thd->org_status_var.ha_update_count);
+  user_stats->rows_stats.key_read_hit+= (thd->status_var.ha_read_key_count -
+                                         thd->org_status_var.ha_read_key_count -
+                                         (thd->status_var.ha_read_key_miss -
+                                          thd->org_status_var.ha_read_key_miss));
+  user_stats->rows_stats.key_read_miss+=   (thd->status_var.ha_read_key_miss -
+                                            thd->org_status_var.ha_read_key_miss);
   user_stats->select_commands+= thd->select_commands;
   user_stats->update_commands+= thd->update_commands;
   user_stats->other_commands+=  thd->other_commands;
@@ -781,6 +788,10 @@ void update_global_user_stats(THD *thd, bool create_user, time_t now)
 bool thd_init_client_charset(THD *thd, uint cs_number)
 {
   CHARSET_INFO *cs;
+
+  // Test a non-default collation ID. See also comments in this function below.
+  DBUG_EXECUTE_IF("thd_init_client_charset_utf8mb3_bin", cs_number= 83;);
+
   /*
    Use server character set and collation if
    - opt_character_set_client_handshake is not set
@@ -803,6 +814,25 @@ bool thd_init_client_charset(THD *thd, uint cs_number)
                cs->cs_name.str);
       return true;
     }
+    /*
+      Some connectors (e.g. JDBC, Node.js) can send non-default collation IDs
+      in the handshake packet, to set @@collation_connection right during
+      handshake. Although this is a non-documenting feature,
+      for better backward compatibility with such connectors let's:
+      a. resolve only default collations according to @@character_set_collations
+      b. preserve non-default collations as is
+
+      Perhaps eventually we should change (b) also to resolve non-default
+      collations accoding to @@character_set_collations. Clients that used to
+      send a non-default collation ID in the handshake packet will have to set
+      @@character_set_collations instead.
+    */
+    if (cs->state & MY_CS_PRIMARY)
+    {
+      Sql_used used;
+      cs= global_system_variables.character_set_collations.
+            get_collation_for_charset(&used, cs);
+    }
     thd->org_charset= cs;
     thd->update_charset(cs,cs,cs);
   }
@@ -1079,7 +1109,6 @@ static int check_connection(THD *thd)
     statistic_increment(aborted_connects_preauth, &LOCK_status);
     return 1; /* The error is set by alloc(). */
   }
-
   auth_rc= acl_authenticate(thd, 0);
   if (auth_rc == 0 && connect_errors != 0)
   {
@@ -1134,7 +1163,7 @@ void setup_connection_thread_globals(THD *thd)
     1    error
 */
 
-bool login_connection(THD *thd)
+static bool login_connection(THD *thd)
 {
   NET *net= &thd->net;
   int error= 0;
@@ -1147,6 +1176,7 @@ bool login_connection(THD *thd)
   my_net_set_write_timeout(net, connect_timeout);
 
   error= check_connection(thd);
+  thd->session_tracker.sysvars.mark_all_as_changed(thd);
   thd->protocol->end_statement();
 
   if (unlikely(error))
@@ -1362,7 +1392,7 @@ void do_handle_one_connection(CONNECT *connect, bool put_in_cache)
   THD *thd;
   if (!(thd= connect->create_thd(NULL)))
   {
-    connect->close_and_delete(0);
+    connect->close_and_delete();
     return;
   }
 
@@ -1387,6 +1417,16 @@ void do_handle_one_connection(CONNECT *connect, bool put_in_cache)
   thd->thr_create_utime= thr_create_utime;
   /* We need to set this because of time_out_user_resource_limits */
   thd->start_utime= thr_create_utime;
+
+  /*
+    handle_one_connection() is normally the only way a thread would
+    start and would always be on the very high end of the stack ,
+    therefore, the thread stack always starts at the address of the
+    first local variable of handle_one_connection, which is thd. We
+    need to know the start of the stack so that we could check for
+    stack overruns.
+  */
+  thd->thread_stack= (char*) &thd;
   setup_connection_thread_globals(thd);
 
   for (;;)
@@ -1427,7 +1467,7 @@ end_thread:
     if (!(connect->create_thd(thd)))
     {
       /* Out of resources. Free thread to get more resources */
-      connect->close_and_delete(0);
+      connect->close_and_delete();
       break;
     }
     delete connect;
@@ -1456,11 +1496,9 @@ end_thread:
   Close connection without error and delete the connect object
   This and close_with_error are only called if we didn't manage to
   create a new thd object.
-
-  Note: err can be 0 if unknown/not inportant
 */
 
-void CONNECT::close_and_delete(uint err)
+void CONNECT::close_and_delete()
 {
   DBUG_ENTER("close_and_delete");
 
@@ -1474,11 +1512,7 @@ void CONNECT::close_and_delete(uint err)
   vio_type= VIO_CLOSED;
 
   --*scheduler->connection_count;
-
-  if (err == ER_CON_COUNT_ERROR)
-    statistic_increment(connection_errors_max_connection, &LOCK_status);
-  else
-    statistic_increment(connection_errors_internal, &LOCK_status);
+  statistic_increment(connection_errors_internal, &LOCK_status);
   statistic_increment(aborted_connects,&LOCK_status);
 
   delete this;
@@ -1502,7 +1536,7 @@ void CONNECT::close_with_error(uint sql_errno,
     delete thd;
     set_current_thd(0);
   }
-  close_and_delete(close_error);
+  close_and_delete();
 }
 
 
