@@ -226,7 +226,7 @@ static bool record_should_be_deleted(THD *thd, TABLE *table, SQL_SELECT *sel,
                                      Explain_delete *explain, bool truncate_history)
 {
   explain->tracker.on_record_read();
-  thd->inc_examined_row_count(1);
+  thd->inc_examined_row_count();
   if (table->vfield)
     (void) table->update_virtual_fields(table->file, VCOL_UPDATE_FOR_DELETE);
   if (!sel || sel->skip_record(thd) > 0)
@@ -482,6 +482,12 @@ bool Sql_cmd_delete::delete_from_single_table(THD *thd)
                                       (uchar *) 0);
   }
 
+  if (conds && optimizer_flag(thd, OPTIMIZER_SWITCH_SARGABLE_CASEFOLD))
+  {
+    conds= conds->top_level_transform(thd, &Item::varchar_upper_cmp_transformer,
+                                          (uchar *) 0);
+  }
+
 #ifdef WITH_PARTITION_STORAGE_ENGINE
   if (prune_partitions(thd, table, conds))
   {
@@ -494,12 +500,6 @@ bool Sql_cmd_delete::delete_from_single_table(THD *thd)
     if (thd->binlog_for_noop_dml(transactional_table))
       DBUG_RETURN(1);
 
-    if (!thd->lex->current_select->leaf_tables_saved)
-    {
-      thd->lex->current_select->save_leaf_tables(thd);
-      thd->lex->current_select->leaf_tables_saved= true;
-    }
-
     my_ok(thd, 0);
     DBUG_RETURN(0);
   }
@@ -671,7 +671,7 @@ bool Sql_cmd_delete::delete_from_single_table(THD *thd)
       if (!(file_sort= filesort(thd, table, &fsort, fs_tracker)))
         goto got_error;
 
-      thd->inc_examined_row_count(file_sort->examined_rows);
+      thd->ps_report_examined_row_count();
       /*
         Filesort has already found and selected the rows we want to delete,
         so we don't need the where clause
@@ -814,17 +814,13 @@ bool Sql_cmd_delete::delete_from_single_table(THD *thd)
                                               delete_history);
     if (delete_record)
     {
-      void *save_bulk_param= thd->bulk_param;
-      thd->bulk_param= nullptr;
       if (!delete_history && table->triggers &&
           table->triggers->process_triggers(thd, TRG_EVENT_DELETE,
                                             TRG_ACTION_BEFORE, FALSE))
       {
         error= 1;
-        thd->bulk_param= save_bulk_param;
         break;
       }
-      thd->bulk_param= save_bulk_param;
 
       // no LIMIT / OFFSET
       if (returning && result->send_data(returning->item_list) < 0)
@@ -854,26 +850,23 @@ bool Sql_cmd_delete::delete_from_single_table(THD *thd)
 
       if (likely(!error))
       {
-	deleted++;
-	thd->bulk_param= nullptr;
+        deleted++;
         if (!delete_history && table->triggers &&
             table->triggers->process_triggers(thd, TRG_EVENT_DELETE,
                                               TRG_ACTION_AFTER, FALSE))
         {
           error= 1;
-          thd->bulk_param= save_bulk_param;
           break;
         }
-        thd->bulk_param= save_bulk_param;
-	if (!--limit && using_limit)
-	{
-	  error= -1;
-	  break;
-	}
+        if (!--limit && using_limit)
+        {
+          error= -1;
+          break;
+        }
       }
       else
       {
-	table->file->print_error(error,
+        table->file->print_error(error,
                                  MYF(thd->lex->ignore ? ME_WARNING : 0));
         if (thd->is_error())
         {
@@ -925,7 +918,6 @@ cleanup:
   {
     thd->lex->current_select->save_leaf_tables(thd);
     thd->lex->current_select->leaf_tables_saved= true;
-    thd->lex->current_select->first_cond_optimization= false;
   }
 
   delete deltempfile;
@@ -964,7 +956,7 @@ cleanup:
 
       if (log_result > 0)
       {
-	error=1;
+        error=1;
       }
     }
   }
@@ -976,6 +968,8 @@ cleanup:
     if (thd->lex->analyze_stmt)
       goto send_nothing_and_leave;
 
+    thd->collect_unit_results(0, deleted);
+
     if (returning)
       result->send_eof();
     else
@@ -1855,5 +1849,6 @@ bool Sql_cmd_delete::execute_inner(THD *thd)
     delete result;
   }
 
+  status_var_add(thd->status_var.rows_sent, thd->get_sent_row_count());
   return res;
 }
