@@ -583,17 +583,16 @@ bool trans_rollback_stmt(THD *thd)
   DBUG_RETURN(FALSE);
 }
 
-/* Find a named savepoint in the current transaction. */
-static SAVEPOINT **
-find_savepoint(THD *thd, LEX_CSTRING name)
+/** Find a savepoint by name in a savepoint list */
+SAVEPOINT** find_savepoint_in_list(THD *thd,
+                                   const Lex_ident_savepoint name,
+                                   SAVEPOINT ** const list)
 {
-  SAVEPOINT **sv= &thd->transaction->savepoints;
+  SAVEPOINT **sv= list;
 
   while (*sv)
   {
-    if (system_charset_info->strnncoll(
-                     (uchar *) name.str, name.length,
-                     (uchar *) (*sv)->name, (*sv)->length) == 0)
+    if (name.streq(Lex_cstring((*sv)->name, (*sv)->length)))
       break;
     sv= &(*sv)->prev;
   }
@@ -601,6 +600,43 @@ find_savepoint(THD *thd, LEX_CSTRING name)
   return sv;
 }
 
+/* Find a named savepoint in the current transaction. */
+static SAVEPOINT **
+find_savepoint(THD *thd, Lex_ident_savepoint name)
+{
+  return find_savepoint_in_list(thd, name, &thd->transaction->savepoints);
+}
+
+SAVEPOINT* savepoint_add(THD *thd, Lex_ident_savepoint name, SAVEPOINT **list,
+                         int (*release_old)(THD*, SAVEPOINT*))
+{
+  DBUG_ENTER("savepoint_add");
+
+  SAVEPOINT **sv= find_savepoint_in_list(thd, name, list);
+
+  SAVEPOINT *newsv;
+
+  if (*sv) /* old savepoint of the same name exists */
+  {
+    newsv= *sv;
+    if (release_old){
+      int error= release_old(thd, *sv);
+      if (error)
+        DBUG_RETURN(NULL);
+    }
+    *sv= (*sv)->prev;
+  }
+  else if ((newsv= (SAVEPOINT *) alloc_root(&thd->transaction->mem_root,
+                                            savepoint_alloc_size)) == NULL)
+  {
+    my_error(ER_OUT_OF_RESOURCES, MYF(0));
+    DBUG_RETURN(NULL);
+  }
+
+  newsv->name= strmake_root(&thd->transaction->mem_root, name.str, name.length);
+  newsv->length= (uint)name.length;
+  DBUG_RETURN(newsv);
+}
 
 /**
   Set a named transaction savepoint.
@@ -614,7 +650,6 @@ find_savepoint(THD *thd, LEX_CSTRING name)
 
 bool trans_savepoint(THD *thd, LEX_CSTRING name)
 {
-  SAVEPOINT **sv, *newsv;
   DBUG_ENTER("trans_savepoint");
 
   if (!(thd->in_multi_stmt_transaction_mode() || thd->in_sub_stmt) ||
@@ -624,23 +659,12 @@ bool trans_savepoint(THD *thd, LEX_CSTRING name)
   if (thd->transaction->xid_state.check_has_uncommitted_xa())
     DBUG_RETURN(TRUE);
 
-  sv= find_savepoint(thd, name);
+  SAVEPOINT *newsv= savepoint_add(thd, Lex_ident_savepoint(name),
+                                  &thd->transaction->savepoints,
+                                  ha_release_savepoint);
 
-  if (*sv) /* old savepoint of the same name exists */
-  {
-    newsv= *sv;
-    ha_release_savepoint(thd, *sv);
-    *sv= (*sv)->prev;
-  }
-  else if ((newsv= (SAVEPOINT *) alloc_root(&thd->transaction->mem_root,
-                                            savepoint_alloc_size)) == NULL)
-  {
-    my_error(ER_OUT_OF_RESOURCES, MYF(0));
+  if (newsv == NULL)
     DBUG_RETURN(TRUE);
-  }
-
-  newsv->name= strmake_root(&thd->transaction->mem_root, name.str, name.length);
-  newsv->length= (uint)name.length;
 
   /*
     if we'll get an error here, don't add new savepoint to the list.
@@ -688,7 +712,7 @@ bool trans_savepoint(THD *thd, LEX_CSTRING name)
 bool trans_rollback_to_savepoint(THD *thd, LEX_CSTRING name)
 {
   int res= FALSE;
-  SAVEPOINT *sv= *find_savepoint(thd, name);
+  SAVEPOINT *sv= *find_savepoint(thd, Lex_ident_savepoint(name));
   DBUG_ENTER("trans_rollback_to_savepoint");
 
   if (sv == NULL)
@@ -743,7 +767,7 @@ bool trans_rollback_to_savepoint(THD *thd, LEX_CSTRING name)
 bool trans_release_savepoint(THD *thd, LEX_CSTRING name)
 {
   int res= FALSE;
-  SAVEPOINT *sv= *find_savepoint(thd, name);
+  SAVEPOINT *sv= *find_savepoint(thd, Lex_ident_savepoint(name));
   DBUG_ENTER("trans_release_savepoint");
 
   if (sv == NULL)
