@@ -37,9 +37,6 @@ simple headers.
 /* Forward declarations */
 class THD;
 class Field;
-struct dict_table_t;
-struct dict_foreign_t;
-struct table_name_t;
 
 // JAN: TODO missing features:
 #undef MYSQL_FT_INIT_EXT
@@ -86,7 +83,7 @@ innobase_invalidate_query_cache(
 void
 innobase_quote_identifier(
 	FILE*		file,
-	const trx_t*	trx,
+	trx_t*		trx,
 	const char*	id);
 
 /** Quote an standard SQL identifier like tablespace, index or column name.
@@ -96,7 +93,7 @@ Return the string as an std:string object.
 @return a std::string with id properly quoted. */
 std::string
 innobase_quote_identifier(
-	const trx_t*	trx,
+	trx_t*		trx,
 	const char*	id);
 
 /*****************************************************************//**
@@ -142,15 +139,6 @@ at least ENUM and SET, and unsigned integer types are 'unsigned types'
 uint8_t
 get_innobase_type_from_mysql_type(unsigned *unsigned_flag, const Field *field);
 
-/******************************************************************//**
-Compares NUL-terminated UTF-8 strings case insensitively.
-@return 0 if a=b, <0 if a<b, >1 if a>b */
-int
-innobase_strcasecmp(
-/*================*/
-	const char*	a,	/*!< in: first string to compare */
-	const char*	b);	/*!< in: second string to compare */
-
 /** Strip dir name from a full path name and return only the file name
 @param[in]	path_name	full path name
 @return file name or "null" if no file name */
@@ -178,12 +166,6 @@ innobase_convert_from_id(
 	const char*	from,	/*!< in: identifier to convert */
 	ulint		len);	/*!< in: length of 'to', in bytes;
 				should be at least 3 * strlen(to) + 1 */
-/******************************************************************//**
-Makes all characters in a NUL-terminated UTF-8 string lower case. */
-void
-innobase_casedn_str(
-/*================*/
-	char*	a);	/*!< in/out: string to put in lower case */
 
 #ifdef WITH_WSREP
 ulint wsrep_innobase_mysql_sort(int mysql_type, uint charset_number,
@@ -243,15 +225,6 @@ thd_lock_wait_timeout(
 	THD*	thd);	/*!< in: thread handle, or NULL to query
 			the global innodb_lock_wait_timeout */
 
-/******************************************************************//**
-compare two character string case insensitively according to their charset. */
-int
-innobase_fts_text_case_cmp(
-/*=======================*/
-	const void*	cs,		/*!< in: Character set */
-	const void*	p1,		/*!< in: key */
-	const void*	p2);		/*!< in: node */
-
 /******************************************************************//**
 Returns true if transaction should be flagged as read-only.
 @return true if the thd is marked as read-only */
@@ -409,20 +382,23 @@ innobase_convert_to_filename_charset(
 	const char*	from,	/* in: identifier to convert */
 	ulint		len);	/* in: length of 'to', in bytes */
 
-/** Report that a table cannot be decrypted.
-@param thd    connection context
-@param table  table that cannot be decrypted
-@retval DB_DECRYPTION_FAILED (always) */
-ATTRIBUTE_COLD
-dberr_t innodb_decryption_failed(THD *thd, dict_table_t *table);
+/********************************************************************//**
+Helper function to push warnings from InnoDB internals to SQL-layer. */
+void
+ib_push_warning(
+	trx_t*		trx,	/*!< in: trx */
+	dberr_t		error,	/*!< in: error code to push as warning */
+	const char	*format,/*!< in: warning message */
+	...);
 
-/** Report a foreign key error.
-@param error    error to report
-@param name     table name
-@param foreign  constraint */
-ATTRIBUTE_COLD
-void innodb_fk_error(const trx_t *trx, dberr_t err, const char *name,
-                     const dict_foreign_t& foreign);
+/********************************************************************//**
+Helper function to push warnings from InnoDB internals to SQL-layer. */
+void
+ib_push_warning(
+	void*		ithd,	/*!< in: thd */
+	dberr_t		error,	/*!< in: error code to push as warning */
+	const char	*format,/*!< in: warning message */
+	...);
 
 /********************************************************************//**
 Helper function to push warnings from InnoDB internals to SQL-layer. */
@@ -439,11 +415,13 @@ Normalizes a table name string. A normalized name consists of the
 database name catenated to '/' and table name. An example:
 test/mytable. On Windows normalization puts both the database name and the
 table name always to lower case if "set_lower_case" is set to TRUE. */
-void
+size_t
 normalize_table_name_c_low(
 /*=======================*/
 	char*		norm_name,	/*!< out: normalized name as a
 					null-terminated string */
+	size_t		norm_name_size, /*!< in: number of bytes available
+					 in norm_name*/
 	const char*	name,		/*!< in: table name string */
 	bool		set_lower_case); /*!< in: true if we want to set
 					name to lower case */
