@@ -21,6 +21,7 @@
 #include <errno.h>
 
 #include "stdarg.h"
+#include "my_bit.h"
 
 /*
   Returns the number of bytes required for strnxfrm().
@@ -175,7 +176,6 @@ int my_strnncollsp_simple(CHARSET_INFO * cs, const uchar *a, size_t a_length,
 {
   const uchar *map= cs->sort_order, *end;
   size_t length;
-  int res;
 
   end= a + (length= MY_MIN(a_length, b_length));
   while (a < end)
@@ -183,7 +183,6 @@ int my_strnncollsp_simple(CHARSET_INFO * cs, const uchar *a, size_t a_length,
     if (map[*a++] != map[*b++])
       return ((int) map[a[-1]] - (int) map[b[-1]]);
   }
-  res= 0;
   if (a_length != b_length)
   {
     int swap= 1;
@@ -197,15 +196,14 @@ int my_strnncollsp_simple(CHARSET_INFO * cs, const uchar *a, size_t a_length,
       a_length= b_length;
       a= b;
       swap= -1;                                 /* swap sign of result */
-      res= -res;
     }
     for (end= a + a_length-length; a < end ; a++)
     {
       if (map[*a] != map[' '])
-	return (map[*a] < map[' ']) ? -swap : swap;
+        return (map[*a] < map[' ']) ? -swap : swap;
     }
   }
-  return res;
+  return 0;
 }
 
 
@@ -348,7 +346,6 @@ void my_hash_sort_simple_nopad(CHARSET_INFO *cs,
   register const uchar *sort_order=cs->sort_order;
   const uchar *end= key + len;
   register ulong m1= *nr1, m2= *nr2;
-  DBUG_ASSERT(key); /* Avoid UBSAN nullptr-with-offset */
   for (; key < (uchar*) end ; key++)
   {
     MY_HASH_ADD(m1, m2, (uint) sort_order[(uint) *key]);
@@ -365,7 +362,6 @@ void my_hash_sort_simple(CHARSET_INFO *cs,
   register const uchar *sort_order=cs->sort_order;
   const uchar *end;
   uint16 space_weight= sort_order[' '];
-  DBUG_ASSERT(key); /* Avoid UBSAN nullptr-with-offset */
 
   /*
     Remove all trailing characters that are equal to space.
@@ -455,7 +451,11 @@ long my_strntol_8bit(CHARSET_INFO *cs,
     else if (c>='A' && c<='Z')
       c = c - 'A' + 10;
     else if (c>='a' && c<='z')
+    {
       c = c - 'a' + 10;
+      if (base > 36)
+        c += 26;
+    }
     else
       break;
     if (c >= base)
@@ -550,7 +550,11 @@ ulong my_strntoul_8bit(CHARSET_INFO *cs,
     else if (c>='A' && c<='Z')
       c = c - 'A' + 10;
     else if (c>='a' && c<='z')
+    {
       c = c - 'a' + 10;
+      if (base > 36)
+        c += 26;
+    }
     else
       break;
     if (c >= base)
@@ -638,7 +642,11 @@ longlong my_strntoll_8bit(CHARSET_INFO *cs __attribute__((unused)),
     else if (c>='A' && c<='Z')
       c = c - 'A' + 10;
     else if (c>='a' && c<='z')
+    {
       c = c - 'a' + 10;
+      if (base > 36)
+        c += 26;
+    }
     else
       break;
     if (c >= base)
@@ -660,8 +668,12 @@ longlong my_strntoll_8bit(CHARSET_INFO *cs __attribute__((unused)),
 
   if (negative)
   {
-    if (i  > (ulonglong) LONGLONG_MIN)
+    if (i >= (ulonglong) LONGLONG_MIN)
+    {
+      if (i == (ulonglong) LONGLONG_MIN)
+        return LONGLONG_MIN;
       overflow = 1;
+    }
   }
   else if (i > (ulonglong) LONGLONG_MAX)
     overflow = 1;
@@ -735,7 +747,11 @@ ulonglong my_strntoull_8bit(CHARSET_INFO *cs,
     else if (c>='A' && c<='Z')
       c = c - 'A' + 10;
     else if (c>='a' && c<='z')
+    {
       c = c - 'a' + 10;
+      if (base > 36)
+        c += 26;
+    }
     else
       break;
     if (c >= base)
@@ -761,10 +777,7 @@ ulonglong my_strntoull_8bit(CHARSET_INFO *cs,
     return (~(ulonglong) 0);
   }
 
-  /* Avoid undefinite behavior - negation of LONGLONG_MIN */
-  return negative && (longlong) i != LONGLONG_MIN ?
-        -((longlong) i) :
-         (longlong) i;
+  return (negative ? -((longlong) i) : (longlong) i);
 
 noconv:
   err[0]= EDOM;
@@ -1491,8 +1504,6 @@ static my_bool
 my_cset_init_8bit(struct charset_info_st *cs, MY_CHARSET_LOADER *loader)
 {
   cs->state|= my_8bit_charset_flags_from_data(cs);
-  cs->caseup_multiply= 1;
-  cs->casedn_multiply= 1;
   cs->pad_char= ' ';
   if (!cs->to_lower || !cs->to_upper || !cs->m_ctype || !cs->tab_to_uni)
     return TRUE;
@@ -1946,13 +1957,27 @@ my_bool my_propagate_complex(CHARSET_INFO *cs __attribute__((unused)),
 }
 
 
+void my_ci_set_strength(struct charset_info_st *cs, uint strength)
+{
+  DBUG_ASSERT(strength > 0);
+  DBUG_ASSERT(strength <= MY_STRXFRM_NLEVELS);
+  cs->levels_for_order= ((1 << strength) - 1);
+}
+
+
+void my_ci_set_level_flags(struct charset_info_st *cs, uint flags)
+{
+  DBUG_ASSERT(flags < (1<<MY_STRXFRM_NLEVELS));
+  cs->levels_for_order= flags;
+}
+
 /*
   Normalize strxfrm flags
 
   SYNOPSIS:
     my_strxfrm_flag_normalize()
+    cs       - the CHARSET_INFO pointer
     flags    - non-normalized flags
-    nlevels  - number of levels
     
   NOTES:
     If levels are omitted, then 1-maximum is assumed.
@@ -1963,8 +1988,9 @@ my_bool my_propagate_complex(CHARSET_INFO *cs __attribute__((unused)),
     normalized flags
 */
 
-uint my_strxfrm_flag_normalize(uint flags, uint maximum)
+uint my_strxfrm_flag_normalize(CHARSET_INFO *cs, uint flags)
 {
+  uint maximum= my_bit_log2_uint32(cs->levels_for_order) + 1;
   DBUG_ASSERT(maximum >= 1 && maximum <= MY_STRXFRM_NLEVELS);
   
   /* If levels are omitted, then 1-maximum is assumed*/
@@ -2117,8 +2143,6 @@ MY_CHARSET_HANDLER my_charset_8bit_handler=
     my_mb_wc_8bit,
     my_wc_mb_8bit,
     my_mb_ctype_8bit,
-    my_caseup_str_8bit,
-    my_casedn_str_8bit,
     my_caseup_8bit,
     my_casedn_8bit,
     my_snprintf_8bit,
@@ -2137,7 +2161,9 @@ MY_CHARSET_HANDLER my_charset_8bit_handler=
     my_well_formed_char_length_8bit,
     my_copy_8bit,
     my_wc_mb_bin, /* native_to_mb */
-    my_wc_to_printable_8bit
+    my_wc_to_printable_8bit,
+    my_casefold_multiply_1,
+    my_casefold_multiply_1
 };
 
 MY_COLLATION_HANDLER my_collation_8bit_simple_ci_handler =
@@ -2150,12 +2176,13 @@ MY_COLLATION_HANDLER my_collation_8bit_simple_ci_handler =
     my_strnxfrmlen_simple,
     my_like_range_simple,
     my_wildcmp_8bit,
-    my_strcasecmp_8bit,
     my_instr_simple,
     my_hash_sort_simple,
     my_propagate_simple,
     my_min_str_8bit_simple,
-    my_max_str_8bit_simple
+    my_max_str_8bit_simple,
+    my_ci_get_id_generic,
+    my_ci_get_collation_name_generic
 };
 
 
@@ -2169,10 +2196,11 @@ MY_COLLATION_HANDLER my_collation_8bit_simple_nopad_ci_handler =
     my_strnxfrmlen_simple,
     my_like_range_simple,
     my_wildcmp_8bit,
-    my_strcasecmp_8bit,
     my_instr_simple,
     my_hash_sort_simple_nopad,
     my_propagate_simple,
     my_min_str_8bit_simple_nopad,
-    my_max_str_8bit_simple
+    my_max_str_8bit_simple,
+    my_ci_get_id_generic,
+    my_ci_get_collation_name_generic
 };
